@@ -44,6 +44,7 @@
 #include <mail/em-folder-tree.h>
 
 #include <libekolab/kolab-data-imap-account.h>
+#include <libekolab/kolab-util-backend.h>
 #include <libekolabutil/kolab-util-camel.h>
 
 #include "e-kolab-folder-metadata.h"
@@ -208,10 +209,47 @@ kolab_folder_prop_ui_response_cb (GObject *dialog,
 /*----------------------------------------------------------------------------*/
 /* internal statics (UI) */
 
+static gboolean
+kolab_plugin_ui_get_selected_source (EShellView *shell_view,
+                                     ESource **selected_source)
+{
+	ESource *source = NULL;
+	gchar *uri = NULL;
+	EShellSidebar *shell_sidebar = NULL;
+	ESourceSelector *selector = NULL;
+
+	g_return_val_if_fail (shell_view != NULL, FALSE);
+	g_return_val_if_fail (selected_source == NULL || *selected_source == NULL, FALSE);
+
+	shell_sidebar = e_shell_view_get_shell_sidebar (shell_view);
+	g_return_val_if_fail (shell_sidebar != NULL, FALSE);
+
+	g_object_get (shell_sidebar, "selector", &selector, NULL);
+	g_return_val_if_fail (selector != NULL, FALSE);
+
+	source = e_source_selector_peek_primary_selection (selector);
+	uri = source ? e_source_get_uri (source) : NULL;
+	/* check whether we have a Kolab ESource selected */
+	if (uri && g_str_has_prefix (uri, KOLAB_CAMEL_URI_PREFIX))
+		source = g_object_ref (source);
+	else
+		source = NULL;
+
+	g_free (uri);
+	g_object_unref (selector);
+
+	if (selected_source)
+		*selected_source = source;
+	else if (source)
+		g_object_unref (source);
+
+	return source != NULL;
+}
+
 static gchar*
-kolab_plugin_ui_get_selected_path (EShellView *shell_view,
-                                   gboolean *is_kolab_account_node,
-                                   gboolean *is_kolab_folder_node)
+kolab_plugin_ui_path_from_mail_view (EShellView *shell_view,
+                                     gboolean *is_kolab_account_node,
+                                     gboolean *is_kolab_folder_node)
 {
 	EShellSidebar *shell_sidebar = NULL;
 	EMFolderTree *folder_tree = NULL;
@@ -240,6 +278,81 @@ kolab_plugin_ui_get_selected_path (EShellView *shell_view,
 	return selected_path;
 }
 
+static gchar*
+kolab_plugin_ui_path_from_pim_view (EShellView *shell_view,
+                                    gboolean *is_kolab_account_node,
+                                    gboolean *is_kolab_folder_node)
+{
+	gchar *selected_path = NULL;
+	ESource *source = NULL;
+	gboolean is_source = FALSE;
+	gchar *uri = NULL;
+	gchar *proto = NULL;
+
+	g_assert (E_IS_SHELL_VIEW (shell_view));
+
+	*is_kolab_account_node = FALSE;
+	*is_kolab_folder_node = FALSE;
+
+	is_source = kolab_plugin_ui_get_selected_source (shell_view,
+	                                                 &source);
+	if (! is_source)
+		goto exit;
+
+	uri = e_source_get_uri (source);
+	if (uri == NULL)
+		goto exit;
+
+	proto = kolab_util_backend_get_protocol_from_uri (uri);
+	if (g_strcmp0 (proto, KOLAB_CAMEL_PROVIDER_PROTOCOL) != 0) {
+		g_warning ("%s()[%u] got wrong protocol '%s'",
+		           __func__, __LINE__, proto);
+		goto exit;
+	}
+
+	selected_path = kolab_util_backend_get_relative_path_from_uri (uri);
+	if (selected_path == NULL)
+		g_warning ("%s()[%u] selected path is NULL", __func__, __LINE__);
+	*is_kolab_folder_node = TRUE;
+
+ exit:
+	if (uri != NULL)
+		g_free (uri);
+	if (proto != NULL)
+		g_free (proto);
+
+	return selected_path;
+}
+
+static gchar*
+kolab_plugin_ui_get_selected_path (EShellView *shell_view,
+                                   gboolean *is_kolab_account_node,
+                                   gboolean *is_kolab_folder_node)
+{
+	gchar *selected_path = NULL;
+	const gchar *view_name = NULL;
+
+	g_assert (E_IS_SHELL_VIEW (shell_view));
+
+	/* TODO find a cleaner solution for this.
+	 *
+	 * Since it seems we cannot directly check for
+	 * the EShellView subtype, we use the view name
+	 * property to tell the types apart...
+	 */
+	view_name = e_shell_view_get_name (shell_view);
+
+	if (g_strcmp0 (view_name, "mail") == 0)
+		selected_path = kolab_plugin_ui_path_from_mail_view (shell_view,
+		                                                     is_kolab_account_node,
+		                                                     is_kolab_folder_node);
+	else
+		selected_path = kolab_plugin_ui_path_from_pim_view (shell_view,
+		                                                    is_kolab_account_node,
+		                                                    is_kolab_folder_node);
+	return selected_path;
+}
+
 static GtkWidget*
 kolab_plugin_ui_selected_folder_widget (GtkAction *action,
                                         EShellView *shell_view)
@@ -256,8 +369,10 @@ kolab_plugin_ui_selected_folder_widget (GtkAction *action,
 	foldername = kolab_plugin_ui_get_selected_path (shell_view,
 	                                                &is_kolab_account_node,
 	                                                &is_kolab_folder_node);
-	g_assert (!is_kolab_account_node);
-	g_assert (is_kolab_folder_node);
+
+	g_return_val_if_fail (foldername != NULL, NULL);
+	g_return_val_if_fail (!is_kolab_account_node, NULL);
+	g_return_val_if_fail (is_kolab_folder_node, NULL);
 
 	label = gtk_label_new (NULL);
 	labeltext = g_strconcat ("<b>", _("Selected Folder"), ": </b>", foldername, NULL);
@@ -280,7 +395,23 @@ kolab_plugin_ui_action_kolab_properties_cb (GtkAction *action,
 
 	uidata = kolab_folder_prop_ui_data_new ();
 
-	uidata->widgets->dialog = \
+	uidata->widgets->selected_folder =
+		kolab_plugin_ui_selected_folder_widget (action, shell_view);
+	if (uidata->widgets->selected_folder == NULL) {
+		/* TODO this should not happen.
+		 *
+		 * If the selected folder is NULL here, it means
+		 * it is no Kolab folder node. There should not
+		 * be a context menu entry in this case in the
+		 * first place.
+		 */
+		g_warning ("%s()[%u]: Selected folder is no Kolab folder node",
+		           __func__, __LINE__);
+		kolab_folder_prop_ui_data_free (uidata);
+		return;
+	}
+
+	uidata->widgets->dialog =
 		gtk_dialog_new_with_buttons (_("Edit Kolab Folder Properties..."),
 		                             NULL, /* parent */
 		                             GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -289,9 +420,6 @@ kolab_plugin_ui_action_kolab_properties_cb (GtkAction *action,
 		                             NULL);
 	gtk_window_set_resizable (GTK_WINDOW (uidata->widgets->dialog), FALSE);
 
-	uidata->widgets->selected_folder = \
-		kolab_plugin_ui_selected_folder_widget (action, shell_view);
-
 	uidata->meta_ui_data = e_kolab_folder_metadata_ui_new ();
 	uidata->perm_ui_data = e_kolab_folder_permissions_ui_new ();
 	/* FIXME get account data here */
@@ -341,43 +469,6 @@ kolab_plugin_ui_action_kolab_properties_cb (GtkAction *action,
 	gtk_widget_show (uidata->widgets->dialog);
 }
 
-static gboolean
-kolab_plugin_ui_get_selected_source (EShellView *shell_view,
-                                     ESource **selected_source)
-{
-	ESource *source;
-	gchar *uri = NULL;
-	EShellSidebar *shell_sidebar;
-	ESourceSelector *selector = NULL;
-
-	g_return_val_if_fail (shell_view != NULL, FALSE);
-	g_return_val_if_fail (selected_source == NULL || *selected_source == NULL, FALSE);
-
-	shell_sidebar = e_shell_view_get_shell_sidebar (shell_view);
-	g_return_val_if_fail (shell_sidebar != NULL, FALSE);
-
-	g_object_get (shell_sidebar, "selector", &selector, NULL);
-	g_return_val_if_fail (selector != NULL, FALSE);
-
-	source = e_source_selector_peek_primary_selection (selector);
-	uri = source ? e_source_get_uri (source) : NULL;
-	/* check whether we have a Kolab ESource selected */
-	if (uri && g_str_has_prefix (uri, KOLAB_CAMEL_URI_PREFIX))
-		source = g_object_ref (source);
-	else
-		source = NULL;
-
-	g_free (uri);
-	g_object_unref (selector);
-
-	if (selected_source)
-		*selected_source = source;
-	else if (source)
-		g_object_unref (source);
-
-	return source != NULL;
-}
-
 static void
 kolab_plugin_ui_enable_actions (GtkActionGroup *action_group,
                                 const GtkActionEntry *entries,
