@@ -183,22 +183,18 @@ test_kolab_util_cal_freebusy (void)
 
 	/* create KolabUtilHttpJob */
 	job = kolab_util_http_job_new ();
+	job->passwd = password;
 	job->pkcs11pin = pkcs11pin;
 
 	/* create CamelURL object */
 	job->url = camel_url_new (url_string, 0);
 	g_assert (job->url != NULL);
 
-	/* FIXME rewrite this for CamelSettings */
-	g_error ("%s: FIXME rewrite this for CamelSettings", __func__);
-#if 0
+	/* set username for basic auth */
 	camel_url_set_user (job->url, username);
-	camel_url_set_passwd (job->url, password);
-#endif
 
 	g_free (url_string);
 	g_free (username);
-	g_free (password);
 
 	/* job.buffer must be initialized NULL (no storage allocated!) */
 	job->buffer = NULL;
