@@ -108,22 +108,18 @@ test_kolab_util_http (void)
 
 	/* create KolabHttpJob */
 	job = kolab_util_http_job_new ();
+	job->passwd = password;
 	job->pkcs11pin = pkcs11pin;
 
 	/* create CamelURL object */
 	job->url = camel_url_new (url_string, 0);
 	g_assert (job->url != NULL);
 
-	/* FIXME rewrite this for CamelSettings */
-	g_error ("%s: FIXME rewrite this for CamelSettings", __func__);
-#if 0
+	/* set username for basic auth */
 	camel_url_set_user (job->url, username);
-	camel_url_set_passwd (job->url, password);
-#endif
 
 	g_free (url_string);
 	g_free (username);
-	g_free (password);
 
 	/* open filestream */
 	file_stream = camel_kolab_stream_new_filestream (outfilename,
@@ -141,7 +137,13 @@ test_kolab_util_http (void)
 	job->buffer = g_byte_array_new ();
 
 	/* issue HTTP GET request */
-	nbytes = kolab_util_http_get (job, NULL);
+	nbytes = kolab_util_http_get (job, &tmp_err);
+	if (tmp_err != NULL) {
+		g_warning ("%s: %s",
+		           __func__, tmp_err->message);
+		g_error_free (tmp_err);
+		tmp_err = NULL;
+	}
 	g_debug ("%s: %d bytes read", __func__, nbytes);
 
 	/* write data to file */
